@@ -11,25 +11,26 @@
  */
 #include <config.h>
 #include "linc-compat.h"
-#include <dirent.h>
 #include <linc/linc-protocol.h>
 #include <linc/linc-connection.h>
 
 #include "linc-private.h"
 
-#undef LOCAL_DEBUG
+#include <glib/gstdio.h>
 
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
 
-#ifdef G_OS_WIN32
-#  include <io.h>
-#  define mkdir(path, mode) _mkdir (path)
+#ifndef G_OS_WIN32
+#include <net/if.h>
+#include <sys/ioctl.h>
 #endif
 
+#undef LOCAL_DEBUG
+
 static char *link_tmpdir = NULL;
-static gboolean use_local_host = FALSE;
+static LinkNetIdType use_local_host = LINK_NET_ID_IS_FQDN;
 
 /*
  * make_local_tmpdir:
@@ -47,13 +48,13 @@ make_local_tmpdir (const char *dirname)
 {
 	struct stat statbuf;
 		
-	if (mkdir (dirname, 0700) != 0) {
+	if (g_mkdir (dirname, 0700) != 0) {
 		int e = errno;
 			
 		switch (e) {
 		case 0:
 		case EEXIST:
-			if (stat (dirname, &statbuf) != 0)
+			if (g_stat (dirname, &statbuf) != 0)
 				g_error ("Can not stat %s\n", dirname);
 
 #if !defined (__CYGWIN__) && !defined(_WIN32)
@@ -73,7 +74,7 @@ make_local_tmpdir (const char *dirname)
 		}
 	}
 
-#if defined (HAVE_UTIME_H) || defined (HAVE_SYS_UTIME_H)
+#if (defined (HAVE_UTIME_H) || defined (HAVE_SYS_UTIME_H)) && !defined (G_OS_WIN32)
 	{ /* Hide some information ( apparently ) */
 		struct utimbuf utb;
 		memset (&utb, 0, sizeof (utb));
@@ -131,28 +132,152 @@ link_get_tmpdir (void)
 #define LINK_RESOLV_UNSET_IPV6
 #endif
 
-
-
 #if defined(AF_INET) || defined(AF_INET6) || defined (AF_UNIX)
+
+static char *
+get_netid(LinkNetIdType which, 
+	  char *buf, 
+	  size_t len)
+{
+	if (LINK_NET_ID_IS_LOCAL == which)
+		return strncpy(buf, "localhost", len);
+
+	if (LINK_NET_ID_IS_IPADDR == which) {
+#ifndef G_OS_WIN32
+		struct sockaddr_in *adr = NULL;
+		struct ifreq my_ifreqs[2];
+		struct ifconf my_ifconf;
+		int num, i, sock;
+
+		my_ifconf.ifc_len = sizeof(my_ifreqs);
+		my_ifconf.ifc_req = my_ifreqs;
+		sock = socket(AF_INET,SOCK_DGRAM,0);
+		if (-1 == sock) 
+			goto out;
+
+		if (ioctl(sock,SIOCGIFCONF,&my_ifconf) < 0) {
+			close(sock);
+			goto out;
+		}
+		close(sock);
+
+		num = my_ifconf.ifc_len / sizeof(struct ifreq);
+		if (!num) 
+			goto out;
+
+ 		for (i = 0; i < num; i++) {
+			adr = (struct sockaddr_in *)&my_ifreqs[i].ifr_ifru.ifru_addr;
+			if (strcmp("127.0.0.1", inet_ntoa(adr->sin_addr)))
+				break;
+		}
+		/* will be 127.0.0.1 anyway, if no other address is defined... */
+		return strncpy(buf, (const char*)inet_ntoa(adr->sin_addr), len);
+#else
+		SOCKET sock;
+		DWORD nbytes;
+		/* Let's hope 20 interfaces is enough. There doesn't
+		 * seem to be any way to get information about how
+		 * many interfaces there are.
+		 */
+		INTERFACE_INFO interfaces[20]; 
+		int i;
+
+		sock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+		if (sock == INVALID_SOCKET)
+			goto out;
+
+		if (WSAIoctl (sock, SIO_GET_INTERFACE_LIST, NULL, 0,
+			      interfaces, sizeof (interfaces),
+			      &nbytes, NULL, NULL) == SOCKET_ERROR ||
+		    nbytes == 0 ||
+		    (nbytes % sizeof (INTERFACE_INFO)) != 0) {
+			closesocket (sock);
+			goto out;
+		}
+		closesocket (sock);
+
+		/* First look for a non-loopback IPv4 address */
+		for (i = 0; i < nbytes / sizeof (INTERFACE_INFO); i++) {
+			if ((interfaces[i].iiFlags & IFF_UP) &&
+			    !(interfaces[i].iiFlags & IFF_LOOPBACK) &&
+			    interfaces[i].iiAddress.Address.sa_family == AF_INET)
+				return strncpy(buf, inet_ntoa(interfaces[i].iiAddress.AddressIn.sin_addr), len);
+		}
+
+		/* Next look for a loopback IPv4 address */
+		for (i = 0; i < nbytes / sizeof (INTERFACE_INFO); i++) {
+			if ((interfaces[i].iiFlags & IFF_UP) &&
+			    (interfaces[i].iiFlags & IFF_LOOPBACK) &&
+			    interfaces[i].iiAddress.Address.sa_family == AF_INET)
+				return strncpy(buf, inet_ntoa(interfaces[i].iiAddress.AddressIn.sin_addr), len);
+		}
+
+		/* Fail */
+		goto out;
+#endif
+	}
+
+	if ((LINK_NET_ID_IS_SHORT_HOSTNAME == which) || (LINK_NET_ID_IS_FQDN == which)) {
+		if (gethostname(buf, len))
+			goto out;
+#ifndef G_OS_WIN32
+		if (errno == EINVAL)
+			goto out;
+#endif
+		if (LINK_NET_ID_IS_SHORT_HOSTNAME == which) {
+			char *retv = buf;
+			while (*buf) {
+				if ('.' == *buf)
+					*buf = '\0';
+				buf++;
+			}
+			return retv;
+		}
+	}
+
+	if (LINK_NET_ID_IS_FQDN == which) {
+#ifdef HAVE_GETADDRINFO
+		struct addrinfo *result, hints;
+		memset(&hints, 0, sizeof(struct addrinfo));
+		hints.ai_flags = AI_CANONNAME;
+		if (!getaddrinfo(buf, NULL, &hints, &result)) {
+			strncpy(buf, result->ai_canonname, len);
+			freeaddrinfo(result);
+		} else
+			goto out;
+#else
+		struct hostent *he;
+
+		/* gethostbyname() is MT-safe on Windows, btw */
+		he = gethostbyname(buf);
+
+		if (!he)
+		  goto out;
+
+		strncpy(buf, he->h_name, len);
+#endif
+		return buf;
+	}
+
+out:
+	return NULL;
+}
+
 const char *
 link_get_local_hostname (void)
 {
 	static char local_host[NI_MAXHOST] = { 0 };
 
-	if (use_local_host)
-		return "localhost";
-
-	if (local_host [0])
+	if (local_host[0])
 		return local_host;
 
-	if (gethostname (local_host, NI_MAXHOST) == -1)
-		return NULL;
+	get_netid(use_local_host, local_host, NI_MAXHOST);
 
 	return local_host;
 }
 
 void
-link_use_local_hostname (gboolean use)
+link_use_local_hostname (LinkNetIdType use)
 {
         use_local_host = use;
 }
@@ -667,10 +792,23 @@ link_protocol_get_sockinfo_ipv4 (const LinkProtocolInfo  *proto,
                                       sizeof (struct in_addr), AF_INET);
 		if (!host)
 			return FALSE;
-		else
-			hname = host->h_name;
+		hname = host->h_name;
 	}
-
+#ifdef G_OS_WIN32
+	{
+		/* Make sure looking up that name works */
+		char *hname_copy = g_strdup (hname);
+		host = gethostbyname (hname_copy);
+		
+		g_free (hname_copy);
+		if (host == NULL) {
+			/* Nope, use IP address then */
+			hname = inet_ntoa (sa_in->sin_addr);
+		} else {
+			hname = host->h_name;
+		}
+	}
+#endif
 	return link_protocol_get_sockinfo_ipv46 (hname, sa_in->sin_port, 
 						 hostname, portnum);
 }
@@ -857,7 +995,7 @@ link_protocol_unix_destroy (int         fd,
 			    const char *dummy,
 			    const char *pathname)
 {
-	unlink (pathname);
+	g_unlink (pathname);
 }
 
 static gboolean
@@ -1000,7 +1138,7 @@ link_protocol_destroy_addr (const LinkProtocolInfo *proto,
  *
  * Return Value: an array of #LinkProtocolInfo structures.
  */
-LinkProtocolInfo * const
+LinkProtocolInfo *
 link_protocol_all (void)
 {
 	return static_link_protocols;
@@ -1015,7 +1153,7 @@ link_protocol_all (void)
  * Return Value: a pointer to a valid #LinkProtocolInfo structure if 
  *               the protocol is supported by linc, NULL otherwise.
  */
-LinkProtocolInfo * const
+LinkProtocolInfo *
 link_protocol_find (const char *name)
 {
 	int i;
@@ -1037,7 +1175,7 @@ link_protocol_find (const char *name)
  * Return Value: a pointer to a valid #LinkProtocolInfo structure if
  *               the protocol is supported by linc, NULL otherwise.
  */
-LinkProtocolInfo * const
+LinkProtocolInfo * 
 link_protocol_find_num (const int family)
 {
 	int i;
